#!/bin/env Rscript

args <- commandArgs(TRUE)
verbose     = TRUE
rf          = args[1]
wf          = args[2]
minRestBeta = as.numeric(args[3])
minAllBeta  = as.numeric(args[4])
wc          = as.numeric(args[5])
minRestT    = as.numeric(args[6])
minAllT     = as.numeric(args[7])

#1:799739 2	10 12	29
drf  = read.table(rf,header=FALSE,nrows=wc)
d_   = drf[drf$V4 >= minRestBeta,]
d    = d_[d_$V5 >= minAllBeta,]
rawT = d$V4/d$V5
rawN = d$V2/d$V3

m      = mean(rawT)
v      = var(rawT)
ab     = m*(1-m)/v - 1  # ab = alpha + beta
alpha0 = m*ab
beta0  = ab-alpha0
fn <- function(x) {
 a = d$V4
 b = d$V5 - d$V4
 K = length(a)
 alpha <- x[1]; beta <- x[2]
 sum(log(beta(a + alpha, b + beta))) - K*log(beta(alpha, beta))
}
ptm    <- proc.time()
param  <- optim(c(alpha0, beta0), fn, control=list(fnscale=-1))$par
#print(proc.time() - ptm)

alpha <- param[1]
beta  <- param[2]
cat("Estimated  alpha: ",round(alpha,2),"; Estimated  beta : ",round(beta,2),"\n")

drf$adjT    = (drf$V4+alpha)/(drf$V5+alpha+beta)
drf$adjN    = (drf$V2+alpha)/(drf$V3+alpha+beta)
drf$adjDiff = drf$adjT - drf$adjN

o           = order(drf$adjDiff,decreasing=TRUE)

adjDiff     = round(drf$adjDiff[o],3)
ID          = drf$V1[o]
Nx          = drf$V2[o]
Nn          = drf$V3[o]
Tx          = drf$V4[o]
Tn          = drf$V5[o]
Percentile  = round(100*(1-rank(drf$adjDiff[o])/length(o)),1)

dd          = data.frame(ID,adjDiff,Percentile,Nx,Nn,Tx,Tn)
re_         = dd[dd$Tx >= minRestT,]
out         = re_[re_$Tn >= minAllT,]

write.table(out,file=wf,sep="\t",row.names=FALSE,quote=FALSE,col.names=FALSE)
