#!/usr/bin/perl
use strict;
use warnings;
use Getopt::Long;
use List::MoreUtils qw(uniq);

my ($L,$nVote,$allEB,$restEB,$allT,$restT,$maxT_hR);
$nVote   = 2;
$allEB   = 10;
$restEB  = 3;
$allT    = 10;
$restT   = 2;
$maxT_hR = 0.9;
GetOptions (
  "L=s" => \$L,
  "nVote=i" => \$nVote,
  "allEB=i" => \$allEB,
  "restEB=i" => \$restEB,
  "allT=i" => \$allT,
  "restT=i" => \$restT,
  "maxT_hR=i" => \$maxT_hR,
);

open(L,$L)||die"$!";
my @L = <L>;
close L;
chomp @L;

my $inputEB_CMD = "";
my $EB_CMD = "";
my $EB_R = "./SMUG_BaseSub.EB.R";
my $inputEB = "";
my $outputEB = "";
my $outputEBR = "";
#my @id_diff = ();
my @id_acgt = ();
my %summary_diff = ();
my %summary_acgt = ();
for my $i (0..$#L) {
    my $_EB = $L[$i].".EB.".$restEB.".".$allEB.".ThR".$maxT_hR;
	$inputEB = $_EB.".inputEB";
	$inputEB_CMD = "awk '{if(\$1!~/#/ && \$9<=".$maxT_hR.") print \$1,\$10,\$11}' ".$L[$i]." | sed 's/\\//\\t/g' > $inputEB";
	#print $inputEB_CMD,"\n";
	system $inputEB_CMD;
	$outputEB  = $_EB.".Tumor.".$restT.".".$allT.".txt";
	$outputEBR = $_EB.".R_re";
	$EB_CMD = "wc -l $inputEB | awk '{print \$1}' | xargs -i echo Rscript $EB_R $inputEB $outputEB $restEB $allEB {} $restT $allT | sh > $outputEBR";
	#print $EB_CMD,"\n";
	system $EB_CMD;
	&hashDiff($outputEB,\%summary_diff);
	$id_acgt[$i] = &hashACGT($L[$i],\%summary_acgt);
}
print "\n  calculate EB done!\n";

my $w = $L.".nVote.".$nVote.".EB.".$restEB.".".$allEB.".ThR".$maxT_hR.".Tumor.".$restT.".".$allT.".txt";
open(W,">$w")||die"$!";

#3:180434779	rs121913279	3	0.745	0%	0.248	0.000%	A->A*/G	86,0,0,0->15,0,7,0	112,0,0,0->40,0,12,0	80,0,0,0->19,0,34,0	-	-	-	-	-
print W "#loc\trsID\tsamples\tsumDiff\tsum%\tmeanDiff\tmean%\tgt.N->gt.T";
for my $i (1..@L) {
    print W "\tP".$i.".N->T";
}
print W "\n";

foreach my $k (sort {$summary_diff{$b}->[1] <=> $summary_diff{$a}->[1]} keys %summary_diff) {
    if ($summary_diff{$k}->[0] >= $nVote) {
        print W $k,"\t",$summary_acgt{$k}->[1],"\t",$summary_diff{$k}->[0],"\t",$summary_diff{$k}->[1],"\t",$summary_diff{$k}->[2],"%";
        my $n1 = sprintf("%.3f",$summary_diff{$k}->[1]/$summary_diff{$k}->[0]);
        my $n2 = sprintf("%.3f",$summary_diff{$k}->[2]/$summary_diff{$k}->[0]);
        print W "\t",$n1,"\t",$n2,"%";
        my $acgtK = $summary_acgt{$k}->[0];
        print W "\t",join(";",@$acgtK);               # ;;;
        for my $i (0..$#id_acgt) {
            if (exists $id_acgt[$i]->{$k}) {
                print W "\t",$id_acgt[$i]->{$k}->[1];
            } else {
                print W "\t","-";
            }
        }
        print W "\n";
    }
}
close W;
print "\n  write $w done!\n";


sub hashDiff {
    my ($f,$summary) = @_;
    open(F,$f)||die"$!";
    my @li = ();
    #my %diff = ();
    while (<F>) {
        chomp $_;
        @li = split(/\t/,$_); # 10:105820185	0.327	0	0	35	18	30
        #my @diff = ($li[1],$li[2],$li[5],$li[6]);
        #$diff{$li[0]} = \@diff;
        if (exists $summary->{$li[0]}) {
            $summary->{$li[0]}->[0] += 1;
            $summary->{$li[0]}->[1] += $li[1];
            $summary->{$li[0]}->[2] += $li[2];
        } else {
            $summary->{$li[0]}->[0] = 1;       # no. of samples
            $summary->{$li[0]}->[1] = $li[1];  # sum of diff
            $summary->{$li[0]}->[2] = $li[2];  # sum of percentage
        }
    }
    close F;
    #return \%diff;
}

sub hashACGT {
    my ($f,$summary) = @_;
    open(F,$f)||die"$!";
    my @li = ();
    my $id = "";
    my %acgt = ();
    while (<F>) {
        chomp $_;
        @li = split(/\t/,$_); # 16:32071654	rs62042679	A	%	A*/G	-100	134.64	0	0.4	0/20	6/15	20,0,0,0	9,0,6,0
        $id = $li[1];
        if ($li[3] eq '.' || $li[3] eq '%') {
            $li[3] = $li[2];
        }
        if ($li[4] eq '.' || $li[4] eq '%') {
            $li[4] = $li[2];
        }
        my $a1 = $li[3]."->".$li[4];
        my $a2 = $li[11]."->".$li[12];
        my @acgt = ($a1,$a2);
        $acgt{$li[0]} = \@acgt;
        
        if (exists $summary->{$li[0]}) {
            my $gt = $summary->{$li[0]}->[0];
            push(@$gt,$a1);
            my @gt = uniq(@$gt);
            $summary->{$li[0]}->[0] = \@gt;
        } else {
            my @gt = ();
            push(@gt,$a1);
            $summary->{$li[0]}->[0] = \@gt; # genotype of normal and tumor N->T
            $summary->{$li[0]}->[1] = $id;  # id rs# or .
        }
    }
    close F;
    return \%acgt;
}

